#ifndef	_FLIPIO_H
#define	_FLIPIO_H
/*****************************************************************************
* PROJECT:	Sangoma Frame Relay Network Driver
* FILE:		flipio.h	Driver API Definitions
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* Copyright (c) 1995-1996 Sangoma Technologies Inc.  All Rights Reserved.
* ============================================================================
* Rev.#	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  1.20	Dec.18,1996	GK	added 'signalling' fild to flipPortCfg_t.
*  1.10	Jun.06,1996	GK	Updated for SDLA Driver 3.0 compatibility.
*  1.00	Mar.11,1996	GK	Beta release.
*  0.22	Feb.12,1996	GK	Removed FLIP_RELEASE definition.
*  0.10	Nov.21,1995	GK	Uses socket-type IOCTL, still in alpha.
*  0.01	Oct.12,1995	GK	Initial version. (Alpha release).
*****************************************************************************/

#include <linux/sockios.h>		/* for the SIOCDEVPRIVATE, etc. */

/*
 * Driver Version Info.
 */
#define	FLIP_VERSION	1		/* driver major version number */
#define	FLIP_SIGNATURE	0x50494C46L	/* 'FLIP' in low-byte oder */

/*
 * Driver IOCTL Commands (up to 16).
 */
#define	FLIPIO_CONFIG	(SIOCDEVPRIVATE + 0)	/* configure port */
#define	FLIPIO_RESET	(SIOCDEVPRIVATE + 1)	/* reset port */
#define	FLIPIO_STATUS	(SIOCDEVPRIVATE + 2)	/* get port status */
#define	FLIPIO_ADD	(SIOCDEVPRIVATE + 3)	/* add interface */
#define	FLIPIO_DELETE	(SIOCDEVPRIVATE + 4)	/* delete interface */
#define	FLIPIO_IFSTATUS	(SIOCDEVPRIVATE + 5)	/* get interface status */

/*
 * Error Codes.
 */
#define	FLIPER_BADCMD	-1	/* invalid command (general) */
#define	FLIPER_BADARG	-2	/* invalid argument (general) */
#define	FLIPER_BADVER	-3	/* invalid driver version (general) */
#define	FLIPER_NEXIST	-4	/* port does not exist (general) */
#define	FLIPER_DOWN	-5	/* adapter is down (CONFIG) */
#define	FLIPER_BADFW	-6	/* invalid firmware (CONFIG) */
#define	FLIPER_BADSTATE	-7	/* invalid state (CONFIG/RESET) */
#define	FLIPER_MALLOC	-8	/* memory allocation failure (ADD) */
#define	FLIPER_REGDEV	-9	/* cannot register device (ADD) */
#define	FLIPER_IFEXIST	-10	/* interface already exists (ADD) */
#define	FLIPER_IFNEXIST	-11	/* interface does not exist (DELETE) */

#define	FLIPER_BADPORT	-12	/* invalid port number */
#define	FLIPER_BADDLCI	-13	/* invalid DLCI */
#define	FLIPER_DLCIBUSY	-14	/* dlci is in use */

/*
 * Miscellaneous.
 */
#define	DLCI_NUM	24	/* max number of DLCIs per Access Node */

/****** Data Structures *****************************************************/

/*----------------------------------------------------------------------------
 * Port Configuration Info. Passed to FLIPIO_CONFIG IOCTL.
 *----------------------------------------------------------------------------
 */
typedef struct flipPortCfg
{
  unsigned int	port;		/* port number */
  unsigned int	type;		/* port type: RS-232/V.35 */
  unsigned int	clocking;	/* internal/external */
  unsigned int	speed;		/* data transfer rate, bps */
  unsigned int	station;	/* CPE/Access Node */
  unsigned int	signalling;	/* in-channel signalling AnnexD/LMI/Q.933 */
  unsigned int	mtu;		/* maximum transmit unit size */
  unsigned int	T391;		/* Link Integrity Verification timer */
  unsigned int	T392;		/* Polling Verification timer */
  unsigned int	N391;		/* Full Status Polling Cycle counter */
  unsigned int	N392;		/* Error Threshold counter */
  unsigned int	N393;		/* Monitored Events counter */
  unsigned int	CIR;		/* Committed Information Rate */
  unsigned int	dlcinum;	/* number of DLCs (Access Node only) */
  unsigned int	dlci[DLCI_NUM];	/* list of DLCIs (Access Node only) */
} flipPortCfg_t;

/*----------------------------------------------------------------------------
 * Port Status Info. Returned by FLIPIO_STATUS IOCTL.
 *----------------------------------------------------------------------------
 */
typedef struct flipPortSts
{
  unsigned int	ifcnt;		/* number of interfaces on this port */
  unsigned int	rxint_cnt;	/* Rx interrupt count */
  unsigned int	txint_cnt;	/* Tx interrupt count */
  unsigned int	spuri_cnt;	/* spurious interrupt count */
  unsigned int	status;		/* link/modem status */
  unsigned int	dlci_num;	/* number of active DLCIs */
  unsigned int	dlci[DLCI_NUM];	/* list of active DLCIs */
				/*----- link-level statistics --------------*/
  unsigned int	rx_overruns;	/* receive overrun error count */
  unsigned int	rx_aborts;	/* reveived aborted frames */
  unsigned int	rx_bad_crc;	/* receive CRC error count */
  unsigned int	rx_too_long;	/* reveived unexpetedly long frames */
  unsigned int	rx_dropped;	/* misc. frames discarded */
  unsigned int	rx_bad_dlci;	/* frames with invalid DLCI */
  unsigned int	rx_bad_format;	/* frames with invalid format */
  unsigned int	tx_underruns;	/* transmitter underrun count */
  unsigned int	tx_aborts;	/* transmitted aborted frames */
  unsigned int	tx_retries;	/* frames re-transmitted */
  unsigned int	dcd_low_cnt;	/* number of times DCD dropped */
  unsigned int	cts_low_cnt;	/* number of times CTS dropped */
  unsigned int	reserved[4];	/* reserved for future use */
} flipPortSts_t;

/*
 * 'status' defines.
 */
#define	FLIP_MODEM_CTS	0x0001	/* CTS line active */
#define	FLIP_MODEM_DCD	0x0002	/* DCD line active */
#define	FLIP_LINK_UP	0x0010	/* frame relay link active */

/*----------------------------------------------------------------------------
 * Network Interface Configuration Info. Passed to FLIPIO_ADD IOCTL.
 *----------------------------------------------------------------------------
 */
typedef struct flipIfCfg
{
  unsigned int	ifno;		/* interface number */
  unsigned int	port;		/* port number */
  unsigned int	dlci;		/* DLCI number for this interface */
} flipIfCfg_t;

/*----------------------------------------------------------------------------
 * Network Interface Status Info. Returned by  FLIPIO_IFSTATUS IOCTL.
 *----------------------------------------------------------------------------
 */
typedef struct flipIfSts
{
  unsigned int	ifno;		/* interface number */
  unsigned int	port;		/* port number */
  unsigned int	dlci;		/* DLCI number for this interface */
} flipIfSts_t;

/*----------------------------------------------------------------------------
 * User Data Structure (Call-Specific Data).
 *----------------------------------------------------------------------------
 */
typedef union flipUsr
{
  flipPortCfg_t	portcfg;	/* FLIPIO_CONFIG */
  flipPortSts_t	portsts;	/* FLIPIO_STATUS */
  flipIfCfg_t	ifcfg;		/* FLIPIO_ADD */
  flipIfSts_t	ifsts;		/* FLIPIO_IFSTATUS */
  unsigned int	port;		/* FLIPIO_RESET, FLIPIO_STATUS */
  unsigned int	ifno;		/* FLIPIO_DELETE, FLIPIO_IFSTATUS */
} flipUsr_t;

/*----------------------------------------------------------------------------
 * Driver IOCTL Data Structure.
 *----------------------------------------------------------------------------
 */
typedef struct flipCtl
{
  unsigned long		signature;	/* signature constant */
  unsigned short	version;	/* API version number */
  short 		result;		/* return code */
  unsigned short	length;		/* size of the call-spesific data */
  flipUsr_t		u;		/* call-specific data */
} flipCtl_t;

#endif	/* _FLIPIO_H */

