#ifndef	_FR_API_H
#define	_FR_API_H
/*****************************************************************************
* PROJECT:	SDLA Frame Relay Firmware Module
* FILE:		fr_api.h	Firmware-level API definitions.
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* (c) Copyright 1995-1996 Sangoma Technologies Inc.  All Rights Reserved.
* ============================================================================
*  Rev	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  1.00	Apr.29,1996	GK	Merged version (S502 & S508).
*  0.01	Sep.26,1995	GK	Initial version.
*****************************************************************************/

/*----------------------------------------------------------------------------
 * Notes:
 * ------
 * 1. All structures defined in this file are byte-alined.  To ensure
 *    portability of this code between different platforms and compilers, one
 *    of the following defines must be defined before including this file:
 *
 *	Compiler	Platform	Define		Use option
 *	--------	--------	------		----------
 *	GNU C		Linux		_GNUC_		-
 *	Microsoft C	DOS/Windows	_MSC_		-
 *
 * 2. To ensure compatibility with different firmware versions, one of the
 *    following defines must be defined before including this file:
 *
 *	Firmware	Adapter		Define
 *	--------	-------		------
 *	fr502.sfm	S502A/E		_FR502_
 *	fr508.sfm	S508		_FR508_
 */

#ifdef		_GNUC_
#  ifndef	PACKED
#    define	PACKED	__attribute__((packed))
#  endif	/* PACKED */
#else
#  define	PACKED
#endif
#ifdef		_MSC_
#  pragma	pack(1)
#endif

/*
 * Adapter memory layout and important constants.
 */
#if defined	_FR502_
#define	FR_CTL_WINDOW	0xE000	/* base of the control window */
#define	FR_CMD_OFFSET	0xF60	/* offset of the command block */
#define	FR_FLAGS_OFFS	0x1FF0	/* offset of status flags */
#define FR_MAX_DATA	4096	/* maximum data buffer length */
#define	FR_RX_WINDOW	0xA000	/* base of the direct receive window */
#define	FR_RXCTL_OFFS	0x900	/* direct receive control block */

#elif defined	_FR508_
#define	FR_CTL_WINDOW	0xE000	/* base of the control window */
#define	FR_CMD_OFFSET	0	/* offset of the command block */
#define	FR_FLAGS_OFFS	0x1000	/* offset of the status flags */
#define FR_MAX_DATA	4080	/* maximum data buffer length */
#define	FR_TXBUF_INFO	0x1100	/* offset of Tx buffer information */
#define	FR_RXBUF_INFO	0x1120	/* offset of Rx buffer information */
#endif

/****** Data Structures *****************************************************/

/*----------------------------------------------------------------------------
 * Frame Relay Command Block.
 *----------------------------------------------------------------------------
 * Located at offset FR_CMD_OFFSET into FR_CTL_WINDOW.
 * For S502 also at offset FR_RXCTL_OFFS into FR_RX_WINDOW.
 */
typedef struct frCmd
{
  unsigned char	 flag		PACKED;	/* 00h: command execution flag */
  unsigned char	 cmd		PACKED;	/* 01h: command code */
  unsigned short length		PACKED;	/* 02h: length of data buffer */
  unsigned char	 result		PACKED;	/* 04h: return code */
  unsigned short dlci		PACKED;	/* 05h: DLCI number */
  unsigned char	 attr		PACKED;	/* 07h: FECN, BECN, DE and C/R bits */
  unsigned short rxlost_int	PACKED;	/* 08h: frms discarded at int. level */
  unsigned long	 rxlost_apl	PACKED;	/* 0Ah: frms discarded at app. level */
  unsigned char	 reserved[2]	PACKED;	/* 0Eh: reserved for future use */
  unsigned char	 data[0]	PACKED;	/* 10h: variable length data field */
} frCmd_t;

/*
 * 'cmd' field defines.
 */
#define	FR_WRITE		0x01
#define	FR_READ			0x02
#define	FR_ISSUE_IS_FRAME	0x03
#define FR_SET_CONFIG		0x10
#define FR_READ_CONFIG		0x11
#define FR_COMM_DISABLE		0x12
#define FR_COMM_ENABLE		0x13
#define FR_READ_STATUS		0x14
#define FR_READ_STATISTICS	0x15
#define FR_FLUSH_STATISTICS	0x16
#define	FR_LIST_ACTIVE_DLCI	0x17
#define FR_FLUSH_DATA_BUFFERS	0x18
#define	FR_ADD_DLCI		0x20
#define	FR_DELETE_DLCI		0x21
#define	FR_ACTIVATE_DLCI	0x22
#define	FR_DEACTIVATE_DLCI	0x22
#define FR_READ_MODEM_STATUS	0x30
#define FR_SET_MODEM_STATUS	0x31
#define FR_READ_ERROR_STATS	0x32
#define FR_FLUSH_ERROR_STATS	0x33
#define FR_READ_CODE_VERSION	0x40
#define	FR_SET_INTR_FLAGS	0x50
#define	FR_READ_INTR_FLAGS	0x51

/*
 * 'result' field defines.
 */
#define FRRES_OK		0x00	/* command executed successfully */
#define	FRRES_DISABLED		0x01	/* communications not enabled */
#define	FRRES_INOPERATIVE	0x02	/* channel inoperative */
#define	FRRES_DLCI_INACTIVE	0x03	/* DLCI is inactive */
#define	FRRES_DLCI_INVALID	0x04	/* DLCI is not configured */
#define	FRRES_TOO_LONG		0x04
#define	FRRES_TOO_MANY		0x05
#define	FRRES_CIR_OVERFLOW	0x07	/* Tx throughput has exceeded CIR */
#define	FRRES_BUFFER_OVERFLOW	0x08
#define	FRRES_LINK_DOWN		0x10	/* DCD and/or CTS dropped */
#define	FRRES_CHANNEL_DOWN	0x11	/* channel became inoperative */
#define	FRRES_CHANNEL_UP	0x12	/* channel became operative */
#define	FRRES_DLCI_CHANGE	0x13	/* DLCI status (or number) changed */
#define	FRRES_DLCI_MISMATCH	0x14
#define	FRRES_INVALID_CMD	0x1F	/* invalid command */

/*
 * 'attr' field defines.
 */
#define	FRATTR_

/*----------------------------------------------------------------------------
 * Frame Relay Flags.
 *----------------------------------------------------------------------------
 * located at offset FR_FLAGS_OFFS into FR_CTL_WINDOW.
 */
typedef struct	frFlags
{
#if defined	_FR502_
  unsigned char	reserved1[1]	PACKED;	/* 00h: */
  unsigned char	tx_ready	PACKED;	/* 01h: transmit buffer available */
  unsigned char	rx_ready	PACKED;	/* 02h: received frame available */
  unsigned char	event		PACKED;	/* 03h: asynchronous event */
  unsigned char	mdm_status	PACKED;	/* 04h: modem status */
  unsigned char	reserved2[8]	PACKED;	/* 05h: */
  unsigned char	intr_flag	PACKED;	/* 0Dh: interrupt flag */
  unsigned char	intr_mask	PACKED;	/* 0Eh: interrupt mask */
#elif defined	_FR508_
  unsigned char	reserved1[3]	PACKED;	/* 00h: reserved */
  unsigned char	event		PACKED;	/* 03h: asynchronous event */
  unsigned char	mdm_status	PACKED;	/* 04h: modem status */
  unsigned char	reserved2[11]	PACKED;	/* 05h: reserved */
  unsigned char	intr_flag	PACKED;	/* 10h: interrupt flag */
  unsigned char	intr_mask	PACKED;	/* 11h: interrupt mask */
  unsigned long	tse_offs	PACKED;	/* 12h: Tx status element offset */
#endif
} frFlags_t;

/*
 * 'event' field defines.
 */
#define	FR_EVENT_STATUS		0x01	/* channel status change ??? */
#define	FR_EVENT_DLC_STATUS	0x02	/* DLC status change */
#define	FR_EVENT_BAD_DLCI	0x04	/* FSR included wrong DLCI */
#define	FR_EVENT_LINK_DOWN	0x40	/* DCD or CTS low */

/*
 * 'mdm_status' field defines.
 */
#define	FR_MDM_DCD		0x08	/* mdm_status: DCD */
#define	FR_MDM_CTS		0x20	/* mdm_status: CTS */

/*
 * 'intr_flag' & 'intr_mask' fields defines.
 */
#define	FR_INTR_RXRDY		0x01	/* Rx ready */
#define	FR_INTR_TXRDY		0x02	/* Tx ready */
#define	FR_INTR_MODEM		0x04	/* modem status change (DCD, CTS) */
#define	FR_INTR_READY		0x08	/* interface command completed */
#define	FR_INTR_DLC		0x10	/* DLC status change */
#define	FR_INTR_TIMER		0x20	/* millisecond timer */

#ifdef	_FR508_
/*----------------------------------------------------------------------------
 * Receive Buffer Configuration Info. S508 only!
 *----------------------------------------------------------------------------
 * Located at offset FR_RXBUF_INFO into FR_CTL_WINDOW.
 */
typedef struct	frBufInfo
{
  unsigned short rse_num	PACKED;	/* 00h: number of status elements */
  unsigned long	 rse_base	PACKED;	/* 02h: receive status array base */
  unsigned long	 rse_next	PACKED;	/* 06h: next status element */
  unsigned long	 buf_base	PACKED;	/* 0Ah: rotational buffer base */
  unsigned short reserved	PACKED;	/* 0Eh:  */
  unsigned long	 buf_top	PACKED;	/* 10h: rotational buffer top */
} frBufInfo_t;

/*----------------------------------------------------------------------------
 * Buffer Status Element. S508 only!
 *----------------------------------------------------------------------------
 * Array of structures of this type is located at offset defined by 'rse_base'
 * field of frBufInfo_t structure into absolute adapter memory address space.
 */
typedef struct	frBufCtl
{
  unsigned char	 flag		PACKED;	/* 00h: ready flag */
  unsigned short length		PACKED;	/* 01h: frame length */
  unsigned short dlci		PACKED;	/* 03h: DLCI */
  unsigned char	 attr		PACKED;	/* 05h: FECN/BECN/DE/CR */
  unsigned short timestamp	PACKED;	/* 06h: time stamp */
  unsigned short reserved[2]	PACKED; /* 08h:  */
  unsigned long	 offset		PACKED;	/* 0Ch: buffer absolute address */
} frBufCtl_t;
#endif

/*----------------------------------------------------------------------------
 * Global Configuration Block. Passed to FR_SET_CONFIG command when dlci == 0.
 *----------------------------------------------------------------------------
 */
typedef struct	frConf
{
  unsigned short station_type	PACKED;	/* 00h: station type (DCE/DTE) */
  unsigned short conf_flags	PACKED;	/* 02h: configuration bits */
  unsigned short baud_rate	PACKED;	/* 04h: Baud rate in Kbps */
  unsigned short port_mode	PACKED;	/* 06h: clock source, RS232/V35 */
  unsigned short max_txdata	PACKED;	/* 08h: maximum transmit length */
  unsigned short T391		PACKED;	/* 0Ah:  */
  unsigned short T392		PACKED;	/* 0Ch:  */
  unsigned short N391		PACKED;	/* 0Eh:  */
  unsigned short N392		PACKED;	/* 10h:  */
  unsigned short N393		PACKED;	/* 12h:  */
  unsigned short CIR_fwd	PACKED;	/* 14h:  */
  unsigned short Bc_fwd		PACKED;	/* 16h:  */
  unsigned short Be_fwd		PACKED;	/* 18h:  */
  unsigned short CIR_bwd	PACKED;	/* 1Ah:  */
  unsigned short Bc_bwd		PACKED;	/* 1Ch:  */
  unsigned short Be_bwd		PACKED;	/* 1Eh:  */
  unsigned short dlci[0]	PACKED;	/* 20h:  */
} frConf_t;

/*
 * 'station_type' defines.
 */
#define	FRCFG_STATION_CPE	0
#define	FRCFG_STATION_NODE	1

/*
 * 'conf_flags' defines.
 */
#define	FRCFG_IGNORE_TX_CIR	0x0001
#define	FRCFG_IGNORE_RX_CIR	0x0002
#define	FRCFG_DONT_RETRANSMIT	0x0004
#define	FRCFG_IGNORE_CBS	0x0008
#define	FRCFG_THROUGHPUT	0x0010	/* enable throughput calculation */
#define	FRCFG_DIRECT_RX		0x0080	/* enable direct receive buffer */
#define	FRCFG_AUTO_CONFIG	0x8000	/* enable  auto DLCI configuration */

/*
 * 'baud_rate' defines.
 */
#define	FRCFG_BAUD_1200		12
#define	FRCFG_BAUD_2400		24
#define	FRCFG_BAUD_4800		48
#define	FRCFG_BAUD_9600		96
#define	FRCFG_BAUD_19200	19
#define	FRCFG_BAUD_38400	38
#define	FRCFG_BAUD_56000	56
#define	FRCFG_BAUD_64000	64
#define	FRCFG_BAUD_128000	128

/*
 * 'port_mode' defines.
 */
#define	FRCFG_MODE_EXT_CLK	0x0000
#define	FRCFG_MODE_INT_CLK	0x0001
#ifdef	_FR508_
#define	FRCFG_MODE_V35		0x0000
#define	FRCFG_MODE_RS232	0x0002
#endif

/*----------------------------------------------------------------------------
 * Channel Configuration Block. Passed to FR_SET_CONFIG command when dlci != 0
 *----------------------------------------------------------------------------
 */
typedef struct	frDLCConf
{
  unsigned short conf_flags	PACKED;	/* 00h: configuration bits */
  unsigned short CIR_fwd	PACKED;	/* 02h:  */
  unsigned short Bc_fwd		PACKED;	/* 04h:  */
  unsigned short Be_fwd		PACKED;	/* 06h:  */
  unsigned short CIR_bwd	PACKED;	/* 08h:  */
  unsigned short Bc_bwd		PACKED;	/* 0Ah:  */
  unsigned short Be_bwd		PACKED;	/* 0Ch:  */
} frDLCConf_t;

/*----------------------------------------------------------------------------
 * Interrupt generation control block.  Passed to / returned by
 * FR_SET_INTR_FLAGS & FR_READ_INTR_FLAGS commands, respectively.
 *----------------------------------------------------------------------------
 */
typedef struct frIntrCtl
{
  unsigned char	 flags		PACKED;	/* 00h: interrupt enable flags */
  unsigned short tx_len		PACKED;	/* 01h: required Tx buffer size */
#if defined	_FR508_
  unsigned char	 irq		PACKED;	/* 03h: IRQ level to activate */
  unsigned char	 moreflags	PACKED;	/* 04h: ?? */
  unsigned short timeout	PACKED;	/* 05h: ms, for timer interrupt */
#endif
} frIntrCtl_t;

/*----------------------------------------------------------------------------
 * Channel Status Block. Returned by FR_READ_STATUS command.
 *----------------------------------------------------------------------------
 */
typedef struct	frDLCStatus
{
  unsigned char		status	PACKED;	/* 00h: link/DLCI status */
  struct
  {
    unsigned short	dlci	PACKED;	/* 01h: DLCI number */
    unsigned char	status	PACKED;	/* 03h: DLCI status */
  } circuit[0]			PACKED;
} frDLCStatus_t;

/*
 * 'status' defines.
 */
#define	FR_LINK_INOPER	0x00		/* for global status (DLCI == 0) */
#define	FR_LINK_OPER	0x01
#define	FR_DLCI_DELETED	0x01		/* for circuit status (DLCI != 0) */
#define	FR_DLCI_ACTIVE	0x02
#define	FR_DLCI_WAITING	0x04
#define	FR_DLCI_NEW	0x08
#define	FR_DLCI_REPORT	0x40

/*----------------------------------------------------------------------------
 * Link (global) Statistics Block. Returned by FR_READ_STATISTICS command with
 * DCLI == 0.
 *----------------------------------------------------------------------------
 */
typedef struct	frLinkStat
{
  unsigned short rx_too_long	PACKED;	/* 00h:  */
  unsigned short rx_dropped	PACKED;	/* 02h:  */
  unsigned short rx_dropped2	PACKED;	/* 04h:  */
  unsigned short rx_bad_dlci	PACKED;	/* 06h:  */
  unsigned short rx_bad_format	PACKED;	/* 08h:  */
  unsigned short retransmitted	PACKED;	/* 0Ah:  */
  unsigned short cpe_tx_FSE	PACKED;	/* 0Ch:  */
  unsigned short cpe_tx_LIV	PACKED;	/* 0Eh:  */
  unsigned short cpe_rx_FSR	PACKED;	/* 10h:  */
  unsigned short cpe_rx_LIV	PACKED;	/* 12h:  */
  unsigned short node_rx_FSE	PACKED;	/* 14h:  */
  unsigned short node_rx_LIV	PACKED;	/* 16h:  */
  unsigned short node_tx_FSR	PACKED;	/* 18h:  */
  unsigned short node_tx_LIV	PACKED;	/* 1Ah:  */
  unsigned short rx_ISF_err	PACKED;	/* 1Ch:  */
  unsigned short rx_unsolicited	PACKED;	/* 1Eh:  */
  unsigned short rx_SSN_err	PACKED;	/* 20h:  */
  unsigned short rx_RSN_err	PACKED;	/* 22h:  */
  unsigned short T391_timeouts	PACKED;	/* 24h:  */
  unsigned short T392_timeouts	PACKED;	/* 26h:  */
  unsigned short N392_reached	PACKED;	/* 28h:  */
  unsigned short cpe_SSN_RSN	PACKED;	/* 2Ah:  */
  unsigned short current_SSN	PACKED;	/* 2Ch:  */
  unsigned short current_RSN	PACKED;	/* 2Eh:  */
  unsigned short curreny_T391	PACKED;	/* 30h:  */
  unsigned short current_T392	PACKED;	/* 32h:  */
  unsigned short current_N392	PACKED;	/* 34h:  */
  unsigned short current_N393	PACKED;	/* 36h:  */
} frLinkStat_t;

/*----------------------------------------------------------------------------
 * DLCI Statistics Block. Returned by FR_READ_STATISTICS command with
 * DLCI != 0.
 *----------------------------------------------------------------------------
 */
typedef struct	frDLCIStat
{
  unsigned long  tx_frames	PACKED;	/* 00h:  */
  unsigned long  tx_bytes	PACKED;	/* 04h:  */
  unsigned long  rx_frames	PACKED;	/* 08h:  */
  unsigned long  rx_bytes	PACKED;	/* 0Ch:  */
  unsigned long  rx_dropped	PACKED;	/* 10h:  */
  unsigned long  rx_inactive	PACKED;	/* 14h:  */
  unsigned long  rx_exceed_CIR	PACKED;	/* 18h:  */
  unsigned long  rx_DE_set	PACKED;	/* 1Ch:  */
  unsigned long  tx_throughput	PACKED;	/* 20h:  */
  unsigned long  tx_calc_timer	PACKED;	/* 24h:  */
  unsigned long  rx_throughput	PACKED;	/* 28h:  */
  unsigned long  rx_calc_timer	PACKED;	/* 2Ch:  */
} frDLCIStat_t;

/*----------------------------------------------------------------------------
 * Communications Error Statistics Block. Returned by FR_READ_ERROR_STATS
 * command.
 *----------------------------------------------------------------------------
 */
typedef struct	frCommStat
{
  unsigned char  rx_overruns	PACKED;	/* 00h:  */
  unsigned char  rx_bad_crc	PACKED;	/* 01h:  */
  unsigned char  rx_aborts	PACKED;	/* 02h:  */
  unsigned char  rx_too_long	PACKED;	/* 03h:  */
  unsigned char  tx_aborts	PACKED;	/* 04h:  */
  unsigned char  tx_underruns	PACKED;	/* 05h:  */
  unsigned char  tx_missed_undr	PACKED;	/* 06h:  */
  unsigned char  dcd_dropped	PACKED;	/* 07h:  */
  unsigned char  cts_dropped	PACKED;	/* 08h:  */
} frCommStat_t;

/*----------------------------------------------------------------------------
 * Defines for the FR_ISSUE_IS_FRAME command.
 *----------------------------------------------------------------------------
 */
#define	FR_ISF_LVE	2		/* issue Link Verification Enquiry */
#define	FR_ISF_FSE	3		/* issue Full Status Enquiry */

#ifdef		_MSC_
#  pragma	pack()
#endif
#endif	/* _FR_API_H */

