#ifndef	_SDLADRV_H
#define	_SDLADRV_H
/*****************************************************************************
* PROJECT:	Sangoma Synchronous Data-Link Adapter Driver
* FILE:		sdladrv.h	Driver Kernel API Definitions
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* Copyright (c) 1995-1996 Sangoma Technologies Inc.  All Rights Reserved.
* ============================================================================
* Rev.	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  4.00	Nov.11,1996	GK	sdlaData_t changed
*  3.00	Jun.07,1996	GK	sdlaData_t changed
*  2.00	Apr.25,1996	GK	o Changed sdlaStatus structure
*				o Changed IOCTL command codes
*				o Changed card type and firmware codes.
*
*  1.00	Dec.14,1995	GK	Beta version. sdlaBind structure changed.
*  0.03	Nov.13,1995	GK	SDLA types changed
*  0.02	Oct.24,1995	GK	Revision number changed
*  0.01	Oct.14,1995	GK	Alpha version
*****************************************************************************/

/*----------------------------------------------------------------------------
 * Notes:
 * ------
 * 1. To ensure portability of this code between different platforms, one of
 *    the following constants must be defined before including this file:
 *
 *	Platform	Define
 *	--------	------
 *	SCO Unix	_SCO_UNIX_
 *	Linux		_LINUX_
 */

#ifdef	_SCO_UNIX_		/*----- SCO UNIX ---------------------------*/
#include <sys/types.h>		/* useful type definitions */
#include <sys/systm.h>		/* for lbolt, HZ, etc. */
#include <sys/inline.h>		/* for inb(), outb(), etc. */

#elif defined(_LINUX_)		/*----- Linux ------------------------------*/
#include <linux/sched.h>	/* for jiffies, HZ, etc. */
#include <asm/io.h>		/* for inb(), outb(), etc. */
#endif

#include <sdlaio.h>

/****** Macros **************************************************************/

#ifdef	_SCO_UNIX_		/*----- SCO UNIX ---------------------------*/
#define _INB(port)		(inb(port))
#define _OUTB(port, byte)	(outb((port),(byte)))
#define MINOR(dev)		(minor(dev))
#define memcpy(dst, src, cnt)	(bcopy((src),(dst),(cnt)))
#define memclr(ptr, len)	(bzero((ptr),(len)))
#define printk()		(printf())
#define SYS_TIMER		(unsigned long)lbolt
#define SDLA_SLEEP_PRIORITY	(PZERO + 5)	/* sleeping priority */
#define DRV_ENTRY

#elif defined(_LINUX_)		/*----- Linux ------------------------------*/
#define _INB(port)		(inb(port))
#define _OUTB(port, byte)	(outb((byte),(port)))
#define memclr(ptr, len)	(memset((ptr), 0, (len)))
#define SYS_TIMER		(unsigned long)jiffies
#define DRV_ENTRY		static
#endif

#ifndef	min
#define min(a,b)	(((a)<(b))?(a):(b))
#endif

#ifndef	max
#define max(a,b)	(((a)>(b))?(a):(b))
#endif

/****** Defines *************************************************************/

#ifndef	_DEBUG_
#define _DEBUG_	0			/* default debug level */
#endif

#define	SDLA_TIMEOUT	(HZ * 2)	/* SDLA command timeout, in ticks */

/*
 * S50X_ctl() commands.
 */
#define	SDLACTL_INIT	1	/* initialize card: setup DPM, IRQ, etc. */
#define	SDLACTL_DOWN	2	/* shut down card */
#define	SDLACTL_MAPMEM	5	/* map DPM to adapter address space */
#define	SDLACTL_START	6	/* start adapter CPU */
#define	SDLACTL_STOP	7	/* stop CPU */
#define	SDLACTL_INTEN	8	/* enable interrupt generation */
#define	SDLACTL_INTDE	9	/* disable interrupt generation */
#define	SDLACTL_INTACK	10	/* acknowledge PC interrupt */
#define	SDLACTL_INTRPT	11	/* issue interrupt to adapter's CPU */
#define	SDLACTL_GETINFO	12	/* get hardware options */
#define	SDLACTL_DETECT	13	/* detect hardware */

/*
 * SXXX_ctl() return codes.
 */
#define	CTLERR_BADCMD	-1	/* invalid command code */
#define	CTLERR_BADARG	-2	/* argument out of range */
#define	CTLERR_BADSTATE	-3	/* invalid adapter state for requested cmd. */
#define	CTLERR_FATAL	-4	/* hardware failure occured */

/****** Data Structures *****************************************************/

/*----------------------------------------------------------------------------
 * SDLA Hardware Info.
 */
typedef struct sdlaInfo
{
	unsigned hw_opt;		/* hardware options */
	unsigned io_range;		/* I/O port range */
	unsigned long boot_offs;	/* boot code offset */
	unsigned* port;			/* I/O port options list */
	unsigned* irq;			/* IRQ options list */
	unsigned* dpm_base;		/* DPM base options list */
	unsigned* dpm_size;		/* DPM size options list */
	unsigned* clock;		/* CPU clock rate options list */
} sdlaInfo_t;

/*
 * 'hw_opt' defines.
 */
#define	OPT_IRQ_CTL	0x0001	/* IRQ level can be set in software */
#define	OPT_MEM_CTL	0x0002	/* Mem window can be open/closed dynamically */

/*----------------------------------------------------------------------------
 * Module Registration Info.
 */
typedef struct sdlaMod
{
	char name[SDLA_MODNAME_SZ];	/* module name */
	struct sdlaMod* next;		/* link */
	int (*bind)();			/* -> bind routine */
} sdlaMod_t;

/*----------------------------------------------------------------------------
 * Adapter Data Space.
 *	Note: This structure must be kept in sync with sdlaSetup_t and
 *	sdlaStatus_t structures defined in sdlaio.h file!!!
 */
typedef struct sdlaData
{
					/*----- Setup ----------------------*/
	unsigned io_base;		/* adapter I/O port base */
	int irq;			/* interrupt request level */
	unsigned long dpm_base;		/* dual-port memory base */
	unsigned dpm_size;		/* dual-port memory size */
	unsigned clock_rate;		/* adapter clock rate (KHz) */
	unsigned options;		/* configuration options */
					/*----- Run-time status ------------*/
	unsigned type;			/* adapter type */
	unsigned io_range;		/* I/O port range */
	unsigned long mem_size;		/* adapter memory size */
	unsigned long mem_vector;	/* offset of DPM into adapter mem */
	unsigned char regs_wr[SDLA_MAX_RANGE]; /* written to registers */
	unsigned char regs_rd[SDLA_MAX_RANGE]; /* read from registers */
	unsigned state;			/* adapter state */
	unsigned long timestamp;	/* time of the last state change */
	unsigned firmware_id;		/* loaded firmware ID */
	char modname[SDLA_MODNAME_SZ];	/* module name */
	unsigned int_count;		/* interrupt count */
	unsigned max_exetime;		/* maximum command execution time */
	unsigned reserved[11];		/* reserved for future use */
					/*----- Hidden from user -----------*/
	unsigned cardno;		/* card number (0..7) */
	unsigned flags;			/* internal status flags */
	unsigned ioerr;			/* IOCTL error code */
	unsigned long vbase;		/* virtual address of shared memory */
					/*----- Module entry points -------*/
	int (*read)();			/* -> read routine */
	int (*write)();			/* -> write routine */
	int (*ioctl)();			/* -> I/O control routine */
	int (*intr)();			/* -> interrupt routine */
	int (*halt)();			/* -> halt routine */
	int (*unbind)();		/* -> unbind routine */
	void* priv;			/* -> module private data */
} sdlaData_t;

/*
 * 'flags' defines.
 */
#define	FLAG_OPEN	0x8000		/* device open */
#define	FLAG_OEXCL	0x4000		/* device open exclusively */
#define	FLAG_INISR	0x0800		/* inside interrupt routine */
#define	FLAG_INUSE	0x0001		/* DPM window in use */

/*----------------------------------------------------------------------------
 * SDLA Hardware Control Routines.
 */
typedef struct sdlaHwCtl
{
	int (*get_info) (sdlaData_t* ads, sdlaInfo_t* info_p);
	int (*detect) (sdlaData_t* ads);
	int (*init) (sdlaData_t* ads);
	int (*down) (sdlaData_t* ads);
	int (*mapmem) (sdlaData_t* ads, unsigned long addr);
	int (*cpu_start) (sdlaData_t* ads);
	int (*cpu_stop) (sdlaData_t* ads);
	int (*int_enable) (sdlaData_t* ads);
	int (*int_disable) (sdlaData_t* ads);
	int (*int_ack) (sdlaData_t* ads);
	int (*isr) (sdlaData_t* ads);
	int reserved[5];
} sdlaHwCtl_t;

/****** Function Prototypes *************************************************/

extern int sdla_register (unsigned long sig, unsigned ver, sdlaMod_t* mod);
extern int sdla_unregister (char* name);
extern int sdla_exec (sdlaData_t* ads, int offs);
extern int sdla_peek (sdlaData_t* ads, unsigned long addr, void* buf, int len);
extern int sdla_poke (sdlaData_t* ads, unsigned long addr, void* buf, int len);

/*
 * Adapter hardware control routines.
 */
extern int sdla_ctl (sdlaData_t* ads, int cmd, unsigned long lpar);

/****** Global Data *********************************************************/

extern const int SDLA_DEBUG;		/* debug level */
extern const sdlaHwCtl_t sdlactl;	/* SDLA control entry points */

#endif	/* _SDLADRV_H */

