#ifndef	_SDLAIO_H
#define	_SDLAIO_H
/*****************************************************************************
* PROJECT:	Sangoma Synchronous Data-Link Adapter Driver
* FILE:		sdlaio.h	Driver API Definitions
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* Copyright (c) 1995-1996 Sangoma Technologies Inc.  All Rights Reserved.
* ============================================================================
* Rev.#	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  4.00	Nov.11,1996	GK	changed sdlaSetup_t, sdlaStatus_t structures.
*  3.00	May 15,1996	GK	changed IOCTL defines, sdlaXfer_t structure.
*
*  2.00	Apr.25,1996	GK	o Changed sdlaStatus structure
*				o Changed IOCTL command codes
*				o Changed card type and firmware codes.
*
*  1.00	Dec.14,1995	GK	Beta version. sdlaBind structure changed.
*  0.03	Nov.13,1995	GK	SDLA types changed
*  0.02	Oct.24,1995	GK	Revision number changed
*  0.01	Oct.14,1995	GK	Alpha version
*****************************************************************************/

/*
 * Driver Version Info.
 */
#define	SDLA_VERSION	4		/* driver major version number */
#define	SDLA_SIGNATURE	0x414C4453L	/* 'SDLA' in low-byte oder */
#define	SDLA_MAGIC	200		/* IOCTL magic number */
#define	SDLA_IOC_MASK	0xFF		/* IOCTL command mask */
#define	SDLA_IOC(cmd)	((SDLA_MAGIC<<8)+(cmd))

/*
 * Miscellaneous min's, max's & defaults.
 */
#define	SDLA_MAX_UNITS	8		/* max number of cards installed */
#define	SDLA_MAX_RANGE	4		/* max number of I/O registers */
#define	SDLA_WIN_SIZE	0x2000		/* default memory window size */
#define	SDLA_MODNAME_SZ	16		/* max module name length */

/*
 * Adapter Types.
 */
#ifndef	SDLA_TYPES
#  define SDLA_TYPES
#  define SDLA_S502A	5020
#  define SDLA_S502E	5021
#  define SDLA_S503	5030
#  define SDLA_S507	5070
#  define SDLA_S508	5080	
#  define SDLA_S509	5090	
#endif /* SDLA_TYPES */

/*
 * Firmware Identification Numbers.
 *     0 ..  999	Test & Diagnostics
 *  1000 .. 1999	Streaming HDLC
 *  2000 .. 2999	Bisync
 *  3000 .. 3999	SDLC
 *  4000 .. 4999	HDLC
 *  5000 .. 5999	X.25
 *  6000 .. 6999	Frame Relay
 *  7000 .. 7999	PPP
 */
#ifndef	FIRMWARE_IDS
#  define FIRMWARE_IDS
#  define SFID_CALIB502	 200
#  define SFID_STRM502	1200
#  define SFID_STRM508	1800
#  define SFID_BSC502	2200
#  define SFID_BSCMP502	2201
#  define SFID_SDLC502	3200
#  define SFID_HDLC502	4200
#  define SFID_X25_502	5200
#  define SFID_X25_508	5800
#  define SFID_FR502	6200
#  define SFID_FR508	6800
#  define SFID_PPP502	7200
#  define SFID_PPP508	7800
#endif	/* FIRMWARE_IDS */

/*
 * Driver IOCTL Command Codes (up to 255).
 */
#define	SDLAIO_SETUP	(SDLA_IOC(1))	/* re-configure adapter */
#define	SDLAIO_START	(SDLA_IOC(2))	/* start adapter CPU */
#define	SDLAIO_STOP	(SDLA_IOC(3))	/* stop adapter CPU */
#define	SDLAIO_DOWN	(SDLA_IOC(4))	/* disable adapter */
#define	SDLAIO_BIND	(SDLA_IOC(5))	/* bind protocol module */
#define	SDLAIO_UNBIND	(SDLA_IOC(6))	/* unbind protocol module */
#define	SDLAIO_STATUS	(SDLA_IOC(7))	/* get adapter status */
#define	SDLAIO_MAPMEM	(SDLA_IOC(8))	/* map DPM window */
#define	SDLAIO_READ	(SDLA_IOC(9))	/* read block from DPM */
#define	SDLAIO_WRITE	(SDLA_IOC(10))	/* write block to DPM */
#define	SDLAIO_PEEK	(SDLA_IOC(11))	/* absolute block read */
#define	SDLAIO_POKE	(SDLA_IOC(12))	/* absolute block write */

/*
 * Error Codes
 */
#define	SDLAER_BADCMD	-1	/* invalid command */
#define	SDLAER_BADARG	-2	/* invalid argument */
#define	SDLAER_BADVER	-3	/* invalid driver version */
#define SDLAER_CONFLICT	-4	/* conflicting hardware option */
#define	SDLAER_BADTYPE	-5	/* unknown adapter type */
#define	SDLAER_BADHWOPT	-6	/* invalid hardware option */
#define	SDLAER_MEMCONFL	-7	/* DMP area already populated */
#define	SDLAER_NOTFOUND	-8	/* resource not found */
#define	SDLAER_BADMEM	-9	/* memory test failed */
#define	SDLAER_BADSTATE	-10	/* wrong state for requested operation */
#define	SDLAER_DOWN	-11	/* adapter is down */
#define	SDLAER_FAILURE	-12	/* hardware failure */
#define	SDLAER_TIMEOUT	-13	/* firmware timeout */

/*----------------------------------------------------------------------------
 * Adapter Setup Info. Passed to SDLAIO_SETUP command.
 */
typedef struct sdlaSetup
{
	unsigned io_base;		/* adapter I/O port base */
	int irq;			/* interrupt request level */
	unsigned long dpm_base;		/* dual-port memory base */
	unsigned dpm_size;		/* dual-port memory size */
	unsigned clock_rate;		/* adapter clock rate (KHz) */
	unsigned options;		/* configuration options */
} sdlaSetup_t;

/*----------------------------------------------------------------------------
 * Adapter Status Info. Returned by SDLAIO_STATUS command.
 */
typedef struct sdlaStatus
{
					/*----- Setup ----------------------*/
	unsigned io_base;		/* adapter I/O port base */
	int irq;			/* interrupt request level */
	unsigned long dpm_base;		/* dual-port memory base */
	unsigned dpm_size;		/* dual-port memory size */
	unsigned clock_rate;		/* adapter clock rate (KHz) */
	unsigned options;		/* configuration options */
					/*----- Run-time status ------------*/
	unsigned type;			/* adapter type */
	unsigned io_range;		/* I/O port range */
	unsigned long mem_size;		/* adapter memory size */
	unsigned long mem_vector;	/* offset of DPM into adapter mem */
	unsigned char regs_wr[SDLA_MAX_RANGE]; /* written to registers */
	unsigned char regs_rd[SDLA_MAX_RANGE]; /* read from registers */
	unsigned state;			/* adapter state */
	unsigned long timestamp;	/* time of the last state change */
	unsigned firmware_id;		/* loaded firmware ID */
	char modname[SDLA_MODNAME_SZ];	/* module name */
					/*----- Statistics -----------------*/
	unsigned int_count;		/* interrupt count */
	unsigned max_exetime;		/* maximum command execution time */
	unsigned reserved[11];		/* reserved for future use */
} sdlaStatus_t;

/*
 * 'state' defines.
 */
enum SDLA_STATES
{
	SDLA_DISABLED,		/* adapter disabled (shut down) */
	SDLA_CONFIGURED,	/* adapter reset */
	SDLA_RUNNING,		/* adapter is running */
	SDLA_BOUND		/* bound to protocol driver */
};

/*----------------------------------------------------------------------------
 * Adapter start-up info. Passed to SDLAIO_START command.
 */
typedef struct sdlaStart
{
	unsigned firmware_id;
	unsigned entry_point;
	unsigned long dpm_vector;
} sdlaStart_t;

/*----------------------------------------------------------------------------
 * Data Transfer Control Block. Passed to SDLAIO_READ, _WRITE, _PEEK & _POKE.
 */
typedef struct sdlaXfer
{
	unsigned long offset;		/* absolute adapter memory address */
	unsigned long length;		/* block length */
	void* ptr;			/* -> user buffer */
} sdlaXfer_t;

/*----------------------------------------------------------------------------
 * User Data Structure (Call-specific Data).
 */
typedef union sdlaUsr
{
	unsigned char  data[256];	/* limits minimum size */
	sdlaStatus_t status;		/* for SDLAIO_STATUS */
	sdlaSetup_t setup;		/* for SDLAIO_SETUP */
	sdlaStart_t start;		/* for SDLAIO_START */
	sdlaXfer_t xfer;		/* for READ, WRITE, PEEK & POKE */
	unsigned long vector;		/* for SDLAIO_MAPMEM */
	char modname[SDLA_MODNAME_SZ];	/* for SDLAIO_BIND */
} sdlaUsr_t;

/*----------------------------------------------------------------------------
 * Driver IOCTL Data Structure.
 */
typedef struct sdlaCtl
{
	unsigned long signature;	/* signature constant */
	unsigned version;		/* API version number */
	int reserved;			/* reserved for future use */
	int result;			/* return code */
	sdlaUsr_t u;			/* call-specific data */
} sdlaCtl_t;

#endif	/* _SDLAIO_H */

