#ifndef	_SPPPIO_H
#define	_SPPPIO_H
/*****************************************************************************
* PROJECT:	Sangoma Synchronous PPP Network Driver
* FILE:		spppio.h	Driver API Definitions
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* Copyright (c) 1995-1996 Sangoma Technologies Inc.  All Rights Reserved.
* ============================================================================
* Rev.#	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  1.00	May 31,1996	GK	Initial version.
*****************************************************************************/

#include <linux/sockios.h>	/* for the SIOCDEVPRIVATE, etc. */

/*
 * Driver Version Info.
 */
#define	SPPP_VERSION	1		/* driver major version number */
#define	SPPP_SIGNATURE	0x50505053L	/* 'SPPP' in low-byte oder */

/*
 * Network Device IOCTL Command Codes (up to 16).
 */
#define	SPPPIO_EXEC	(SIOCDEVPRIVATE + 0)	/* execute firmware command */
#define	SPPPIO_CONF	(SIOCDEVPRIVATE + 1)	/* configure port */
#define	SPPPIO_INFO	(SIOCDEVPRIVATE + 2)	/* get interface status */
#define	SPPPIO_STAT	(SIOCDEVPRIVATE + 3)	/* get statistics */

/*
 * Error Codes.
 */
#define	SPPPER_BADCMD	-1	/* invalid command (general) */
#define	SPPPER_BADARG	-2	/* invalid argument (general) */
#define	SPPPER_BADVER	-3	/* invalid driver version (general) */
#define	SPPPER_DOWN	-5	/* adapter is down (CONFIG) */
#define	SPPPER_BADSTATE	-7	/* invalid state (CONFIG/RESET) */

/****** Data Structures *****************************************************/

/*----------------------------------------------------------------------------
 * Interface Configuration Block.
 */
typedef struct	spppConf
{
  unsigned int	interface;	/* 0 - RS-232,   1 - V.35 */
  unsigned int	clocking;	/* 0 - external, 1 - internal */
  unsigned long	speed;		/* data transfer rate, bps */
  unsigned int  restart_tmr;	/* restart timer */
  unsigned int  auth_rsrt_tmr;	/* authentication timer */
  unsigned int  auth_wait_tmr;	/* authentication timer */
  unsigned int  mdm_fail_tmr;	/* modem failure timer */
  unsigned int  dtr_drop_tmr;	/* DTR drop timer */
  unsigned int  connect_tmout;	/* connection timeout */
  unsigned int  conf_retry;	/* max. retry */
  unsigned int  term_retry;	/* max. retry */
  unsigned int  fail_retry;	/* max. retry */
  unsigned int  auth_retry;	/* max. retry */
  unsigned int	auth_options;	/* authentication opt. */
  unsigned int	ip_options;	/* IP options */
} spppConf_t;

/*----------------------------------------------------------------------------
 * Interface Status Info. Returned by SPPPIO_INFO IOCTL.
 */
typedef struct spppInfo
{
  int		foo;
} spppInfo_t;

/*----------------------------------------------------------------------------
 * Interface Statistics. Returned by SPPPIO_STAT IOCTL.
 */
typedef struct spppStat
{
  unsigned int	link_status;	/* link/modem status */
  unsigned int	rx_intr_cnt;	/* receive interrupt count */
  unsigned int	tx_intr_cnt;	/* transmit interrupt count */
  unsigned int	ctl_intr_cnt;	/* connection control interrupt count */
  unsigned int	spurious_cnt;	/* spurious interrupt count */
				/*----- Communications statistics ----------*/
  unsigned int	rx_byte_cnt;	/* number of bytes received */
  unsigned int	rx_frm_cnt;	/* number of frames received */
  unsigned int	rx_overruns;	/* receive overrun error count */
  unsigned int	rx_aborts;	/* reveived aborted frames */
  unsigned int	rx_crc_err;	/* receive CRC error count */
  unsigned int	rx_bad_len;	/* frames with invalid length */
  unsigned int	rx_dropped;	/* misc. frames discarded */
  unsigned int	tx_byte_cnt;	/* number of bytes transmitted */
  unsigned int	tx_frm_cnt;	/* number of frames transmitted */
  unsigned int	tx_underruns;	/* transmitter underrun count */
  unsigned int	tx_aborts;	/* transmitted aborted frames */
  unsigned int	tx_retries;	/* frames re-transmitted due to aborts */
  unsigned int	dcd_change;	/* number of times DCD changed state */
  unsigned int	cts_change;	/* number of times CTS changed state */
  unsigned int	reserved[4];	/* reserved for future use */
} spppStat_t;

/*
 * 'link_status' masks.
 */
#define	SPPP_MODEM_CTS	0x0001	/* '1' - CTS line active */
#define	SPPP_MODEM_DCD	0x0002	/* '1' - DCD line active */
#define	SPPP_LINK_UP	0x0100	/* '1' - link is active */

/*----------------------------------------------------------------------------
 * IOCTL Command-Specific Data.
 */
typedef union spppVar
{
  spppConf_t	conf;		/* SPPPIO_CONFIG */
  spppInfo_t	info;		/* SPPPIO_STATUS */
  spppStat_t	stat;		/* SPPPIO_STAT */
} spppVar_t;

/*----------------------------------------------------------------------------
 * Driver IOCTL Data Structure.
 */
typedef struct spppCtl
{
  unsigned long		signature;	/* signature constant */
  unsigned int		version;	/* API version number */
  int	 		result;		/* return code */
  int			reserved;	/* reserved for future use */
  spppVar_t		v;		/* call-specific data */
} spppCtl_t;

#endif	/* _SPPPIO_H */

