#ifndef	_FLIP_H
#define	_FLIP_H
/*****************************************************************************
* PROJECT:	Sangoma Frame Relay Network Device Driver
* FILE:		flip.h		 Common definitions.
* TARGET OS:	Linux
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* Copyright (c) 1995 Sangoma Technologies Inc.  All Rights Reserved.
*
* ============================================================================
*  Rev	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  0.20	Dec.13,1995	GK	Initial version (Alpha release).
*****************************************************************************/

#include <sdladrv.h>

/****** Defines *************************************************************/

#ifndef	PACKED
#define	PACKED		__attribute__((packed))
#endif

#define	DRV_NAME	"flip"		/* device name */

#define	SDLA_MAX_RETRY	10		/* number of retries */
#define	FRRES_TIMEOUT	0xFF		/* firmware timeout error code */
#define	FRRES_FAILURE	0xFE		/* SDLA driver failure error code */
#define	FRRES_MB_INUSE	0xFD		/* SDLA mailbox in use */

/*
 * RFC-1490 Stuff. (Multiprotocol Interconnect over Frame Relay.)
 */
#ifndef	ARPHRD_FR
#define	ARPHRD_FR	0x0F		/* ARP Hardware Type (RFC-1340) */
#endif
#define	FR_HWADDR_LEN	2		/* hardware address length (Q.922) */

typedef	struct Encaps			/* RFC-1490 SNAP Encapsulation */
{
  unsigned char		control	PACKED;	/* Q.922 frame type */
  unsigned char		pad	PACKED;
  unsigned char		nlpid	PACKED;	/* Network Level Protocol ID */
  unsigned char		oui[3]	PACKED;	/* Organizationally Unique Identifier */
  unsigned short	pid	PACKED;	/* protocol identifier */
  unsigned char		data[0]	PACKED;
} Encaps_t;

typedef	struct Encaps2			/* RFC-1490 IP Encapsulation */
{
  unsigned char		control	PACKED;	/* Q.922 frame type */
  unsigned char		nlpid	PACKED;	/* Network Level Protocol ID */
  unsigned char		data[0]	PACKED;
} Encaps2_t;

#define	FLIP_HDR_SZ	(sizeof(Encaps_t))	/* encapsulation header size */

/*
 * Q.922 frame types.
 */
#define	Q922_UI		0x03		/* Unnumbered Info frame */
#define	Q922_XID	0xAF		/* ??? */

/*
 * NLPIDs as per ISO/IEC TR 9577.
 */
#define	NLPID_SNAP	0x80		/* Subnetwork Access Protocol */
#define	NLPID_CLNP	0x81		/* ISO CLNP */
#define	NLPID_ESIS	0x82		/* ISO ESIS */
#define	NLPID_ISIS	0x83		/* ISO ISIS */
#define	NLPID_Q933	0x08		/* CCITT Q.933 */
#define	NLPID_IP	0xCC		/* Internet Protocol Datagram */

#define	DLCI_TO_Q922(d)	((((d)<<6)&0xFC00)|(((d)<<4)&0x00F0))

/****** Data Types **********************************************************/

/*----------------------------------------------------------------------------
 * Physical Port Data Space.
 *----------------------------------------------------------------------------
 */
typedef struct	Port
{
  flipPortCfg_t	cfg;		/* port configuration */
  unsigned int	ifcnt;		/* number of interfaces on this port */
  unsigned int	rxint_cnt;	/* Rx interrupt count */
  unsigned int	txint_cnt;	/* Tx interrupt count */
  unsigned int	spuri_cnt;	/* spurious interrupt count */
  unsigned int	status;		/* link/modem status */
  unsigned int	dlci_num;	/* number of active DLCIs */
  unsigned int	dlci[DLCI_NUM];	/* list of active DLCIs */
				/*----- statistics -------------------------*/
  unsigned int	rx_overruns;	/* receive overrun error count */
  unsigned int	rx_aborts;	/* reveived aborted frames */
  unsigned int	rx_bad_crc;	/* receive CRC error count */
  unsigned int	rx_too_long;	/* reveived unexpetedly long frames */
  unsigned int	rx_dropped;	/* misc. frames discarded */
  unsigned int	rx_bad_dlci;	/* frames with invalid dlci */
  unsigned int	rx_bad_format;	/* frames with invalid format */
  unsigned int	tx_underruns;	/* transmitter underrun count */
  unsigned int	tx_aborts;	/* transmitted aborted frames */
  unsigned int	tx_retries;	/* frames re-transmitted */
  unsigned int	dcd_low_cnt;	/* number of times DCD dropped */
  unsigned int	cts_low_cnt;	/* number of times CTS dropped */
  unsigned int	reserved[4];	/* reserved for future use */
				/*----- private data -----------------------*/
  unsigned int	flags;		/* port status flags */
  sdlaData_t	*ads;		/* -> SDLA data space */
  struct frFunc	*fr_func;	/* -> Frame Relay call table */
  unsigned	wbase;		/* DPM window base for this port */
  unsigned	rsbase;		/* receive status info base */
  unsigned	rslast;		/* receive status info top */
  unsigned	rscrnt;		/* current receive status element */
  unsigned	tscrnt;		/* current transmit status element */
  unsigned	rxbase;		/* receive buffer base (absolute) */
  unsigned	rxtop;		/* receive buffer end (absolute) */
  unsigned int	mb_lock;	/* mailbox lock */
} Port_t;

/*
 * 'flags' defines.
 */
#define	PORT_POLLED	0x0001	/* non interrupt-driven port */
#define	PORT_INTERRUPT	0x1000	/* in interrupt routine */
#define	PORT_READY	0x8000	/* port is operative */

typedef struct	frFunc			/* Frame Relay Function Calls */
{
  int	(*configure)	(Port_t *pds_p);
  int	(*shutdown)	(Port_t *pds_p);
  int	(*send)		(Port_t *pds_p, int dlci, char *buf, int len);
  int	(*interrupt)	(Port_t *pds_p);
  int 	(*status)	(Port_t *pds_p);
  int 	(*statistics)	(Port_t *pds_p);
  int 	(*version)	(Port_t *pds_p, char *str);
} frFunc_t;

/****** Public Data *********************************************************/

extern frFunc_t	fr502_func;
extern frFunc_t	fr508_func;

/****** Public Functions ****************************************************/

int	flip_receive	(Port_t *pds_p, int dlci, char *buf, int len);

#endif	/* _FLIP_H */

