/*****************************************************************************
* PROJECT:	Sangoma Synchronous Data Link Adapter Driver
* FILE:		sdlakapi.c	Platform-independent kernel APIs
* TARGET OS:	SCO Unix, Linux
* AUTHOR:	Gene Kozin
* ----------------------------------------------------------------------------
* Copyright (c) 1995-1996 Sangoma Technologies Inc.  All Rights Reserved.
* ============================================================================
* Rev.#	Date		Who	Details
* -----	-----------	---	----------------------------------------------
*  3.00	Jun.10,1996	GK	
*****************************************************************************/

#include "sdladrv.h"		/* generic SDLA interface */

/****** Kernel Interface Functions ******************************************/

/*============================================================================
 * sdla_peek()		Read absolute adapter memory.
 * ---------------------------------------------------------------------------
 */
int sdla_peek (sdlaData_t *ads_p, unsigned long addr, void *buf, int len)
{
  int	err = 0;

  if (ads_p->state < SDLA_CONFIGURED)		/* verify adapter state */
    err = SDLAER_BADSTATE
  ;
  else if (addr + len > ads_p->mem_size)	/* verify arguments */
    err = SDLAER_BADARG
  ;
  else
  {
    unsigned long oldvec = ads_p->mem_vector;
    unsigned      winsize = ads_p->dpm_size;
    unsigned      curpos, curlen;	/* current offset and block size */
    unsigned long curvec;		/* current DPM window vector */

    /*
     * Transfer data from adapter's memory to data buffer.
     * Note:	it's possible that block length is bigger than DPM window
     *		size, so we must take care of moving DPM window around when
     *		nesessary.
     */
    while (len)
    {
      curpos = addr % winsize;		/* current window offset */
      curvec = addr - curpos;		/* current window vector */
      curlen = (len > (winsize - curpos)) ? (winsize - curpos) : len;

      /*
       * Relocate window and copy block of data.
       */
      sdlactl.mapmem(ads_p, curvec);
      memcpy((void*)buf, (void*)(ads_p->vbase + curpos), curlen);
      addr       += curlen;
      (char*)buf += curlen;
      len        -= curlen;
    }

    /*
     * Restore DPM window position.
     */
    sdlactl.mapmem(ads_p, oldvec);
  }
  return err;
} /* sdla_peek() */

/*============================================================================
 * sdla_poke()		Write Absolute Adapter Memory.
 * ---------------------------------------------------------------------------
 */
int sdla_poke (sdlaData_t *ads_p, unsigned long addr, void *buf, int len)
{
  int	err = 0;

  if (ads_p->state < SDLA_CONFIGURED)		/* verify adapter state */
    err = SDLAER_BADSTATE
  ;
  else if (addr + len > ads_p->mem_size)	/* verify arguments */
    err = SDLAER_BADARG
  ;
  else
  {
    unsigned      winsize = ads_p->dpm_size;
    unsigned      curpos, curlen;	/* current offset and block size */
    unsigned long curvec, oldvec;	/* DPM window vector */

    /*
     * Transfer data from data buffer to adapter's memory.
     * Note:	it's possible that block length is bigger than DPM window
     *		size, so we must take care of moving DPM window around when
     *		nesessary.
     */
    oldvec = ads_p->mem_vector;
    while (len)
    {
      curpos = addr % winsize;	/* current window offset */
      curvec = addr - curpos;		/* current window vector */
      curlen = (len > (winsize - curpos)) ? (winsize - curpos) : len;

      /*
       * Relocate window and copy block of data.
       */
      sdlactl.mapmem(ads_p, curvec);
      memcpy((void *)(ads_p->vbase + curpos), (void *)buf, curlen);
      addr       += curlen;
      (char*)buf += curlen;
      len        -= curlen;
    }

    /*
     * Restore DPM window position.
     */
    sdlactl.mapmem(ads_p, oldvec);
  }
  return err;
} /* sdla_poke() */

/****** End *****************************************************************/
